@@ -177,13 +177,13 @@ ctStanData <- function(ctm, datalong,optimize,sameInitialTimes=FALSE){
       if(length(out)==0) out<-rep(0,ctm$n.manifest)
       if(length(out)<ctm$n.manifest) out<-c(out,rep(0,ctm$n.manifest-length(out)))
       out}) )),nrow=c(nrow(datalong),ncol=ctm$n.manifest)),
-    nordinal_y=array(as.integer(apply(datalong[,ctm$manifestNames,drop=FALSE],1,function(x) 
-      length(x[ctm$manifesttype==2 & x!=99999]))),dim=nrow(datalong)),
-    whichordinal_y=matrix(as.integer(t(apply(datalong[,ctm$manifestNames,drop=FALSE],1,function(x) {
-      out<-as.numeric(which(ctm$manifesttype==2 & x!=99999)) #conditional on whichobs_y
-      if(length(out)==0) out<-rep(0,ctm$n.manifest)
-      if(length(out)<ctm$n.manifest) out<-c(out,rep(0,ctm$n.manifest-length(out)))
-      out}) )),nrow=c(nrow(datalong),ncol=ctm$n.manifest)),
+    # nordinal_y=array(as.integer(apply(datalong[,ctm$manifestNames,drop=FALSE],1,function(x) 
+    #   length(x[ctm$manifesttype==2 & x!=99999]))),dim=nrow(datalong)),
+    # whichordinal_y=matrix(as.integer(t(apply(datalong[,ctm$manifestNames,drop=FALSE],1,function(x) {
+    #   out<-as.numeric(which(ctm$manifesttype==2 & x!=99999)) #conditional on whichobs_y
+    #   if(length(out)==0) out<-rep(0,ctm$n.manifest)
+    #   if(length(out)<ctm$n.manifest) out<-c(out,rep(0,ctm$n.manifest-length(out)))
+    #   out}) )),nrow=c(nrow(datalong),ncol=ctm$n.manifest)),
     ncont_y=array(as.integer(apply(datalong[,ctm$manifestNames,drop=FALSE],1,function(x) 
       length(x[(ctm$manifesttype==0) & x!=99999]))),dim=nrow(datalong)),
     whichcont_y=matrix(as.integer(t(apply(datalong[,ctm$manifestNames,drop=FALSE],1,function(x) {
@@ -221,7 +221,7 @@ ctStanData <- function(ctm, datalong,optimize,sameInitialTimes=FALSE){
       intoverstates=as.integer(ctm$intoverstates),
       verbose=0L,
       manifesttype=array(as.integer(ctm$manifesttype),dim=length(ctm$manifesttype)),
-      nordinal=as.integer(sum(ctm$manifesttype==2)),
+      # nordinal=as.integer(sum(ctm$manifesttype==2)),
       indvaryingindex=array(as.integer(indvaryingindex)),
       intoverpopindvaryingindex=array(as.integer(ctm$intoverpopindvaryingindex)),
       notindvaryingindex=array(as.integer(which(!(1:nparams) %in% indvaryingindex))),
@@ -256,18 +256,18 @@ ctStanData <- function(ctm, datalong,optimize,sameInitialTimes=FALSE){
   standata$tipredsimputedscale <- ctm$tipredsimputedscale
   standata$tipredeffectscale <- ctm$tipredeffectscale
   
-  ##ordinal categories
-  # if(any(ctm$manifesttype==2)){
-  standata$ncategories=array(as.integer(apply(datalong[,ctm$manifestNames,drop=FALSE][,ctm$manifesttype==2,drop=FALSE],2,function(x) length(unique(x)))))
-  standata$nthresholdpars=as.integer(sum(standata$ncategories-1))
-  # } else standata$ncategories=array(integer())
-  if(as.integer(ctm$NOrdinalIntegrationPoints) %% 2 == 0) stop('NOrdinalIntegrationPoints must be odd')
-  standata$nordinalintegrationpoints=as.integer(ctm$NOrdinalIntegrationPoints)
-  midpoint = ceiling(standata$nordinalintegrationpoints/2) #ensure midpoint (mean) is included first, because catprobs is only recomputed for non mean points.
-  standata$sigmapoints=array(round(statmod::gauss.quad.prob(n=ctm$NOrdinalIntegrationPoints,dist='normal')$nodes[
-    c(midpoint,(1:standata$nordinalintegrationpoints)[-midpoint])],5))
-  standata$sigweights=array(round(statmod::gauss.quad.prob(n=ctm$NOrdinalIntegrationPoints,dist='normal')$weights[
-    c(midpoint,(1:standata$nordinalintegrationpoints)[-midpoint])],5))
+  # ##ordinal categories
+  # # if(any(ctm$manifesttype==2)){
+  # standata$ncategories=array(as.integer(apply(datalong[,ctm$manifestNames,drop=FALSE][,ctm$manifesttype==2,drop=FALSE],2,function(x) length(unique(x)))))
+  # standata$nthresholdpars=as.integer(sum(standata$ncategories-1))
+  # # } else standata$ncategories=array(integer())
+  # if(as.integer(ctm$NOrdinalIntegrationPoints) %% 2 == 0) stop('NOrdinalIntegrationPoints must be odd')
+  # standata$nordinalintegrationpoints=as.integer(ctm$NOrdinalIntegrationPoints)
+  # midpoint = ceiling(standata$nordinalintegrationpoints/2) #ensure midpoint (mean) is included first, because catprobs is only recomputed for non mean points.
+  # standata$sigmapoints=array(round(statmod::gauss.quad.prob(n=ctm$NOrdinalIntegrationPoints,dist='normal')$nodes[
+  #   c(midpoint,(1:standata$nordinalintegrationpoints)[-midpoint])],5))
+  # standata$sigweights=array(round(statmod::gauss.quad.prob(n=ctm$NOrdinalIntegrationPoints,dist='normal')$weights[
+  #   c(midpoint,(1:standata$nordinalintegrationpoints)[-midpoint])],5))
   
   
   # #drift, jacobian off diagonal check # seems to work ok internally with expm2
